@@ -107,6 +107,8 @@ struct NvKmsKapiDevice {
     NvKmsDispIOCoherencyModes nisoIOCoherencyModes;
     NvBool supportsSyncpts;
 
+    NvBool coherentGpuMemory;
+
     /* SMG state */
 
     MIGDeviceId migDevice;
@@ -128,11 +130,6 @@ struct NvKmsKapiDevice {
         NvU32 maxCursorSizeInPixels;
 
         NvU8  genericPageKind;
-        NvBool requiresVrrSemaphores;
-
-        NvBool supportsInputColorSpace;
-        NvBool supportsInputColorRange;
-        NvBool supportsWindowMode;
     } caps;
 
     NvU64 supportedSurfaceMemoryFormats[NVKMS_KAPI_LAYER_MAX];
@@ -148,10 +145,6 @@ struct NvKmsKapiDevice {
 
     NvU32 numDisplaySemaphores;
 
-    struct {
-        struct NvKmsMode mode;
-    } headState[NVKMS_KAPI_MAX_HEADS];
-
     struct {
         NvU32 currFlipNotifierIndex;
     } layerState[NVKMS_KAPI_MAX_HEADS][NVKMS_MAX_LAYERS_PER_HEAD];
@@ -177,7 +170,6 @@ struct NvKmsKapiMemory {
 
 struct NvKmsKapiSurface {
     NvKmsSurfaceHandle hKmsHandle;
-    struct NvKmsSize size;
 };
 
 static inline void *nvKmsKapiCalloc(size_t nmem, size_t size)
