@@ -283,8 +283,7 @@ void nvSendHdmiCapsToRm(NVDpyEvoPtr pDpyEvo)
     NVDevEvoPtr pDevEvo = pDispEvo->pDevEvo;
     NvU32 ret;
 
-    if (!pDevEvo->caps.supportsHDMI20 ||
-        nvConnectorUsesDPLib(pDpyEvo->pConnectorEvo)) {
+    if (nvConnectorUsesDPLib(pDpyEvo->pConnectorEvo)) {
         return;
     }
 
@@ -1277,9 +1276,6 @@ void nvDpyUpdateHdmiVRRCaps(NVDpyEvoPtr pDpyEvo)
 
     const NVParsedEdidEvoRec *pParsedEdid = &pDpyEvo->parsedEdid;
     const NVDispEvoRec *pDispEvo = pDpyEvo->pDispEvo;
-    const NVDevEvoRec *pDevEvo = pDispEvo->pDevEvo;
-
-    const NvBool gpuSupportsHDMIVRR = pDevEvo->hal->caps.supportsHDMIVRR;
 
     const NvBool dispSupportsVrr = nvDispSupportsVrr(pDispEvo) && 
                                        !nvkms_conceal_vrr_caps();
@@ -1288,7 +1284,7 @@ void nvDpyUpdateHdmiVRRCaps(NVDpyEvoPtr pDpyEvo)
 
     nvAssert(pParsedEdid->valid);
 
-    if (dispSupportsVrr && gpuSupportsHDMIVRR && (edidVrrMin > 0)) {
+    if (dispSupportsVrr && (edidVrrMin > 0)) {
         if (nvDpyIsAdaptiveSyncDefaultlisted(pDpyEvo)) {
             pDpyEvo->vrr.type =
                 NVKMS_DPY_VRR_TYPE_ADAPTIVE_SYNC_DEFAULTLISTED;
@@ -1687,7 +1683,6 @@ void nvLogEdidCea861InfoEvo(NVDpyEvoPtr pDpyEvo,
 /*
  * HDMI 2.0 4K@60hz uncompressed RGB 4:4:4 (6G mode) is allowed if:
  *
- * - The GPU supports it.
  * - The EDID and NVT_TIMING indicate the monitor supports it, or
  *   this check is overridden.
  */
@@ -1696,9 +1691,6 @@ NvBool nvHdmi204k60HzRGB444Allowed(const NVDpyEvoRec *pDpyEvo,
                                    const NVT_TIMING *pTiming)
 {
     const NVParsedEdidEvoRec *pParsedEdid = &pDpyEvo->parsedEdid;
-    const NVDevEvoRec *pDevEvo = pDpyEvo->pDispEvo->pDevEvo;
-
-    const NvBool gpuSupports444 = pDevEvo->caps.supportsHDMI20;
 
     const NvBool overrideMonitorCheck = ((pParams->overrides &
         NVKMS_MODE_VALIDATION_NO_HDMI2_CHECK) != 0);
@@ -1709,8 +1701,7 @@ NvBool nvHdmi204k60HzRGB444Allowed(const NVDpyEvoRec *pDpyEvo,
 
     nvAssert(pParsedEdid->valid);
 
-    return (gpuSupports444 &&
-            (overrideMonitorCheck || monitorSupports444));
+    return overrideMonitorCheck || monitorSupports444;
 }
 
 /*
@@ -2036,7 +2027,13 @@ NvBool nvHdmiDpySupportsFrl(const NVDpyEvoRec *pDpyEvo)
 {
     const NVDevEvoRec *pDevEvo = pDpyEvo->pDispEvo->pDevEvo;
 
-    nvAssert(nvDpyIsHdmiEvo(pDpyEvo));
+    /*
+     * Can't use FRL if HDMI is not supported by the GPU and the monitor
+     * connection.
+     */
+    if (!nvDpyIsHdmiEvo(pDpyEvo)) {
+        return FALSE;
+    }
 
     /* Can't use FRL if disabled by kernel module param. */
     if (nvkms_disable_hdmi_frl()) {
