@@ -28,13 +28,8 @@
 #include "kernel/gpu_mgr/gpu_mgr.h"
 #include "kernel/gpu/gpu.h"
 #include "kernel/gpu/bus/p2p_api.h"
-#include "kernel/gpu/fifo/kernel_fifo.h"
-#include "gpu/gpu_fabric_probe.h"
 #include "rmapi/rs_utils.h"
 
-static void _knvlinkP2PIdleCallback(OBJGPU *pGpu, void *pArgs);
-void knvlinkABM_WORKITEM(OBJGPU *pGpu, void *pArgs);
-
 NV_STATUS
 knvlinkGetSupportedCounters_GB100
 (
@@ -151,7 +146,7 @@ knvlinkLogAliDebugMessages_GB100
 {
     NV_STATUS status;
     NV2080_CTRL_NVLINK_GET_ERR_INFO_PARAMS *pParams;
-    NvU32 linkMask;
+    NVLINK_BIT_VECTOR linkVec;
     NvU32 failures[7];
     NvU32 failure;
     NvU32 link;
@@ -171,11 +166,11 @@ knvlinkLogAliDebugMessages_GB100
         portMemFree(pParams);
         return status; );
 
-    linkMask = 0x0;
+    bitVectorClrAll(&linkVec);
     failure = 0;
     portMemSet(failures, 0x0, sizeof(failures));
 
-    FOR_EACH_INDEX_IN_MASK(32, link, KNVLINK_GET_MASK(pKernelNvlink, postRxDetLinkMask, 32))
+    FOR_EACH_IN_BITVECTOR(&pKernelNvlink->postRxDetLinkMask, link)
     {
         if ((pParams->linkErrInfo[link].DLStatMN00 & 0xffff) != 0x0)
         {
@@ -187,16 +182,16 @@ knvlinkLogAliDebugMessages_GB100
             if (failure < NV_ARRAY_ELEMENTS(failures))
                 failures[failure++] = pParams->linkErrInfo[link].DLStatMN00;
 
-            linkMask |= NVBIT32(link);
+            bitVectorSet(&linkVec, link);
         }
     }
-    FOR_EACH_INDEX_IN_MASK_END;
+    FOR_EACH_IN_BITVECTOR_END();
 
     if (bFinal)
     {
         nvErrorLog_va((void *)pGpu, ALI_TRAINING_FAIL,
-                      "NVLink: Link training failed for links 0x%x (0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x)\n",
-                      linkMask,
+                      "NVLink: Link training failed for links " NV_BITVECTOR_INLINE_FMTX "(0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x, 0x%x)\n",
+                      NV_BITVECTOR_INLINE_PRINTF_ARG(&linkVec),
                       failures[0],
                       failures[1],
                       failures[2],
@@ -445,91 +440,6 @@ knvlinkValidateFabricBaseAddress_GB100
 
 }
 
-void
-knvlinkABMIdle_WORKITEM
-(
-    NvU32 gpuInstance,
-    void *pArgs
-)
-{
-    OBJGPU *pGpu = gpumgrGetGpu(gpuInstance);
-    KernelNvlink *pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
-    NvU64 enabledLinkMask = knvlinkGetEnabledLinkMask(pGpu, pKernelNvlink);
-
-    if (knvlinkIsP2PActive_IMPL(pGpu, pKernelNvlink))
-    {
-        pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_TRUE);
-        gpuSetRecoveryDrainP2P(pGpu, NV_TRUE);
-        pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_FALSE);
-        return;
-    }
-
-    NV_PRINTF(LEVEL_NOTICE, "GPU%u Detected fabric idle. Applying linkMask 0x%llx and Unmarking Drain P2P.\n",
-            gpuInstance,
-            (enabledLinkMask & pKernelNvlink->pendingAbmLinkMaskToBeReduced));
-
-    // Reuse linkMaskToBeReduced so RBM/ABM flows are the same
-    gpuFabricProbeSetlinkMaskToBeReduced(pGpu->pGpuFabricProbeInfoKernel,
-                                                 pKernelNvlink->pendingAbmLinkMaskToBeReduced);
-
-    pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_TRUE);
-    gpuUnmarkDeviceForDrainP2P(pGpu);
-    pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_FALSE);
-
-    osRemove1HzCallback(pGpu, knvlinkABM_WORKITEM, pArgs);
-}
-
-void
-knvlinkABM_WORKITEM
-(
-    OBJGPU *pGpu,
-    void *pArgs
-)
-{
-    // Queue a work item to check P2P with proper locks
-    NV_STATUS status = osQueueWorkItem(pGpu,
-                                      knvlinkABMIdle_WORKITEM,
-                                      NULL,
-                                      (
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_GPUS |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RO
-                                      ));
-
-    if (status != NV_OK) {
-        NV_PRINTF(LEVEL_ERROR, "Failed to queue P2P idle check.\n");
-    }
-}
-
-NV_STATUS
-knvlinkABMLinkMaskUpdate_GB100
-(
-    OBJGPU *pGpu,
-    KernelNvlink *pKernelNvlink,
-    NvBool bNeedsRCRecovery
-)
-{
-    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
-
-    // Need to kill channels with AMAP of 16 on switch tray removal
-    if (bNeedsRCRecovery)
-    {
-        // Perform PF GFID RC error recovery on all usermode channels.
-        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
-            pRmApi->Control(pRmApi,
-                            pGpu->hInternalClient,
-                            pGpu->hInternalSubdevice,
-                            NV2080_CTRL_CMD_INTERNAL_NVLINK_RC_USER_MODE_CHANNELS,
-                            NULL,
-                            0));
-    }
-
-    // Launch repeated 1Hz workitem to await drainP2P completion and apply link mask
-    (void)osSchedule1HzCallback(pGpu, knvlinkABM_WORKITEM, NULL, NV_OS_1HZ_REPEAT);
-
-    return NV_OK;
-}
-
 /*!
  * @brief   Validates fabric EGM base address.
  *
@@ -566,11 +476,10 @@ knvlinkValidateFabricEgmBaseAddress_GB100
  *
  * @return  NV_TRUE is ENCRYPT_EN is set, else NV_FALSE
  */
-
-NvBool
-knvlinkIsEncryptEnSet_GB100
+NV_STATUS
+knvlinkGetEncryptionBits_GB100
 (
-    OBJGPU *pGpu,
+    OBJGPU       *pGpu,
     KernelNvlink *pKernelNvlink
 )
 {
@@ -584,10 +493,13 @@ knvlinkIsEncryptEnSet_GB100
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Failed to execute RPC to get Nvlink Encrypt Enable Info\n");
-        return NV_FALSE;
+        return status;
     }
 
-    return params.bEncryptEnSet;
+    pKernelNvlink->bMmuNvlinkEncryptEn = params.bMmuNvlinkEncryptEn;
+    pKernelNvlink->bNvlinkTlwEncryptEn = params.bNvlinkTlwEncryptEn;
+
+    return NV_OK;
 }
 
 /*!
@@ -606,20 +518,38 @@ knvlinkIsNvleEnabled_GB100
 {
     NV2080_CTRL_NVLINK_SET_NVLE_ENABLED_STATE_PARAMS params;
     NV_STATUS status;
-    if (!(pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_ENCRYPTION_ENABLED)))
+
+    if (pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_ENCRYPTION_ENABLED))
     {
         //
-        // Nvlink Encryption PDB PROP is set when 
-        // 1. Nvlink Encryption regkey has been enabled AND
-        // 2. Encrypt Enable Bit is set by FSP AND
-        // 3. Secure Scratch Register Bit is set by FSP after reading the NVLE PRC Knob
+        // On MODS, just check PDB_PROP_KNVLINK_ENCRYPTION_ENABLED, on non-MODS platforms, check
+        // the following settings as well
         //
-
-        if (knvlinkIsEncryptEnSet_HAL(pGpu, pKernelNvlink) &&
-            gpuIsNvleModeEnabledInHw_HAL(pGpu)
-            )
+        if (!RMCFG_FEATURE_MODS_FEATURES)
         {
-            pKernelNvlink->setProperty(pKernelNvlink, PDB_PROP_KNVLINK_ENCRYPTION_ENABLED, NV_TRUE);
+            //
+            // Nvlink Encryption PDB PROP is set when 
+            // 1. Nvlink Encryption regkey has been enabled AND
+            // 2. Encrypt Enable Bit is set by FSP AND
+            // 3. Secure Scratch Register Bit is set by FSP after reading the NVLE PRC Knob
+            //
+            if (!(pKernelNvlink->bNvlinkTlwEncryptEn
+                  && (gpuIsNvleModeEnabledInHw_HAL(pGpu) || gpuIsCCEnabledInHw_HAL(pGpu))
+                ))
+            {
+                //
+                // This is an error case, encrypt enable bit and secure scratch register should be set
+                // when CC is enabled.
+                //
+                NV_PRINTF(LEVEL_ERROR,
+                          "CC and Nvlink encryption features are enabled, "
+                          "but encrypt enable bit or PRC knob is not set! Disabling Nvlink encryption\n");
+                pKernelNvlink->setProperty(pKernelNvlink, PDB_PROP_KNVLINK_ENCRYPTION_ENABLED, NV_FALSE);
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_ERROR, "CC and Nvlink encryption features are enabled on the GPU\n");
+            }
         }
     }
 
@@ -712,60 +642,27 @@ knvlinkGetSupportedCoreLinkStateMask_GB100
 }
 
 void
-knvlinkP2PIdleCheck_WORKITEM
-(
-    NvU32 gpuInstance,
-    void *pArgs
-)
-{
-    OBJGPU *pGpu = gpumgrGetGpu(gpuInstance);
-    KernelNvlink *pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
-
-    if (knvlinkIsP2PActive_IMPL(pGpu, pKernelNvlink))
-    {
-        pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_TRUE);
-        gpuSetRecoveryDrainP2P(pGpu, NV_TRUE);
-        pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_FALSE);
-        return;
-    }
-
-    // Invalidate/Suspend probe
-    gpuFabricProbeSuspend(pGpu->pGpuFabricProbeInfoKernel);
-    gpuFabricProbeInvalidate(pGpu->pGpuFabricProbeInfoKernel);
-
-    pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_TRUE);
-    gpuUnmarkDeviceForDrainP2P(pGpu);
-    pGpu->setProperty(pGpu, PDB_PROP_GPU_RECOVERY_SQUASH_XID154, NV_FALSE);
-
-    // Send requested probe
-    NV_ASSERT_OK(gpuFabricProbeResume(pGpu->pGpuFabricProbeInfoKernel));
-
-    osRemove1HzCallback(pGpu, _knvlinkP2PIdleCallback, NULL);
-}
-
-static void
-_knvlinkP2PIdleCallback
+knvlinkP2PIdle_WORKITEM
 (
     OBJGPU *pGpu,
     void *pArgs
 )
 {
-    // Queue a work item to check P2P with proper locks
-    NV_STATUS status = osQueueWorkItem(pGpu,
-                                      knvlinkP2PIdleCheck_WORKITEM,
-                                      NULL,
-                                      (
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_SEMA |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_GPUS |
-                                       OS_QUEUE_WORKITEM_FLAGS_LOCK_API_RO
-                                      ));
+    // TODO: Call p2p idle check (CTK-8435)
 
-    if (status != NV_OK)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Failed to queue P2P idle check.\n");
-    }
+    // Invalidate/Suspend probe
+    gpuFabricProbeSuspend(pGpu->pGpuFabricProbeInfoKernel);
+    gpuFabricProbeInvalidate(pGpu->pGpuFabricProbeInfoKernel);
+
+    // Send requested probe
+    NV_ASSERT_OK(gpuFabricProbeResume(pGpu->pGpuFabricProbeInfoKernel));
+
+    osRemove1HzCallback(pGpu, knvlinkP2PIdle_WORKITEM, pArgs);
 }
 
+/*!
+ * @brief
+ */
 NV_STATUS
 knvlinkTriggerProbeRequest_GB100
 (
@@ -773,7 +670,9 @@ knvlinkTriggerProbeRequest_GB100
     KernelNvlink *pKernelNvlink
 )
 {
-    (void)osSchedule1HzCallback(pGpu, _knvlinkP2PIdleCallback, NULL, NV_OS_1HZ_REPEAT);
+    // TODO: Trigger drainP2P (CTK-8435)
+
+    (void)osSchedule1HzCallback(pGpu, knvlinkP2PIdle_WORKITEM, NULL, NV_OS_1HZ_REPEAT);
 
     return NV_OK;
 }
