@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -176,6 +176,11 @@ subdeviceDestruct_IMPL
     OBJGPU                 *pGpu            = GPU_RES_GET_GPU(pSubdevice);
     NV_STATUS               status          = NV_OK;
 
+    if (pSubdevice == pGpu->pCachedSubdevice)
+    {
+        pGpu->pCachedSubdevice = NULL;
+    }
+
     if (pSubdevice->bGcoffDisallowed)
     {
         osClientGcoffDisallowRefcount(pGpu->pOsGpuInfo, NV_FALSE);
